// SPDX-License-Identifier: MIT

//! Vulnerability processing and matching module

pub use database::{
    Severity, VersionRange, Vulnerability, VulnerabilityDatabase, VulnerabilityMatch,
};
pub use matcher::{DatabaseStats, FixAnalysis, FixSuggestion, MatcherConfig, VulnerabilityMatcher};

pub mod database;
pub mod matcher;
